"""
Easymode is an aspect oriented toolkit that helps making xml based flash
websites easy. The tools included in the toolkit to help you make these kind of
sites include:

* Internationalization of models, with admin support
* Translation of model data using gettext
* Automatic generation of xml from model trees using xslt
* Admin support for model trees with more than 2 levels of related items
* Basic approval support for models

documentation at http://packages.python.org/django-easymode/

release notes at http://packages.python.org/django-easymode/changes.html
"""

__all__ = ('admin', 'debug', 'easypublisher', 'i18n', 'management',
    'templatetags', 'tree', 'utils', 'xslt')
