{% extends "tree/admin/change_form_with_parent_link.html" %}
{% load i18n admin_modify adminmedia %}

{% block object-tools %}
{% if change %}
    {% if not is_popup %}
    <ul class="object-tools">
        {% if has_draft %}
            <li><a href="{% if current %}../drafts/{{has_draft}}/{% else %}drafts/{{has_draft}}/{% endif %}" class="historylink">{% trans "Latest Draft" %}</a></li>
            <li><a href="{% if current %}../drafts/{% else %}drafts/{% endif %}" class="historylink">{% trans "Drafts" %}</a></li>
        {% endif %}
          <li><a href="{% if current %}../history/{% else %}history/{% endif %}" class="historylink">{% trans "History" %}</a></li>
          {% if has_absolute_url %}<li><a href="../../../r/{{ content_type_id }}/{{ object_id }}/" class="viewsitelink">{% trans "View on site" %}</a></li>{% endif%}
          </ul>
        {% endif %}
    {% endif %}
{% endblock %}
